--[[
Name: ZoneInfoClassic
Revision: $Rev: 107 $
Author: Odica; based on the ZoneInfo module of Cartographer 2 by ckknight
SVN: svn://svn.wowace.com/wow/libtourist-3-0/mainline/trunk
Description: Addon that will show levels and other information about the zone being displayed on the World Map.
License: MIT
]]


local function trace(msg)
--	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo: "..msg)
end

if not ZoneInfo then
	ZoneInfo = LibStub("AceAddon-3.0"):NewAddon("ZoneInfoClassic", "AceHook-3.0")
	local ZoneInfo, self = ZoneInfo, ZoneInfo
end

local ZoneInfoDataProviderMixin = CreateFromMixins(MapCanvasDataProviderMixin)
local Tourist = LibStub("LibTouristClassic-1.0")

local t = {}
local showFrame = true
local frameHeight = 0

local GAME_LOCALE = GetLocale()

local translations = {
	koKR = {
		["ZoneInfo"] = "지역 정보",
		["Transports"] = "운송수단",
		["Road connections"] = "연결 지역",
		["Instances"] = "던전",
		["Flight nodes"] = "비행 경로",
		["Fishing"] = "낚시",
		["Herbalism"] = "약초채집",
		["Mining"] = "채광",
		["BG"] = "전장",
		["Raid"] = "공격대",
		["man"] = "인",
	},
	deDE = {
		["ZoneInfo"] = "Zoneninfo",
		["Transports"] = "Transporte",
		["Road connections"] = "Straßenverbindungen",
		["Instances"] = "Instanzen",
		["Flight nodes"] = "Flugknoten",
		["Fishing"] = "Angeln",
		["Herbalism"] = "Kräuterkunde",
		["Mining"] = "Bergbau",
		["BG"] = "SF",
		["Raid"] = "Rauferei",
		["man"] = "Spieler",
	},
	frFR = {
		["ZoneInfo"] = "Infos de zone",
		["Transports"] = "Transports",
		["Road connections"] = "Liaisons routières",
		["Instances"] = "Instances",
		["Flight nodes"] = "Nœuds de vol",
		["Fishing"] = "Pêche",
		["Herbalism"] = "Herboristerie",
		["Mining"] = "Minage",
		["BG"] = "CB",
		["Raid"] = "Raid",
		["man"] = "j",
	},
	esES = {
		["ZoneInfo"] = "Información de la Zona",
		["Transports"] = "Transportes",
		["Road connections"] = "Conexiones por carretera",
		["Instances"] = "Instancias",
		["Flight nodes"] = "Nodos de vuelo",
		["Fishing"] = "Pescar",
		["Herbalism"] = "Herboristería",
		["Mining"] = "Minería",
		["BG"] = "CB",
		["Raid"] = "Raid",
		["man"] = "hombres",
	},
	zhTW = {
		["ZoneInfo"] = "地區資訊",
		["Transports"] = "運輸工具",
		["Road connections"] = "道路連接",
		["Instances"] = "副本",
		["Flight nodes"] = "飛行節點",
		["Fishing"] = "釣魚",
		["Herbalism"] = "草藥學",
		["Mining"] = "採礦",
		["BG"] = "戰場",
		["Raid"] = "襲擊",
		["man"] = "人",
	},
	zhCN = {
		["ZoneInfo"] = "区域信息",
		["Transports"] = "运输",
		["Road connections"] = "道路连接",
		["Instances"] = "副本",
		["Flight nodes"] = "飞行节点",
		["Fishing"] = "钓鱼",
		["Herbalism"] = "草药学",
		["Mining"] = "采矿",
		["BG"] = "战场",
		["Raid"] = "袭击",
		["man"] = "人",
	},
	ruRU = {
		["ZoneInfo"] = "Инфо о Зонах",
		["Transports"] = "Транспорты",
		["Road connections"] = "Дорожные связи",		
		["Instances"] = "Подземелья",
		["Flight nodes"] = "Летные узлы",		
		["Fishing"] = "Ловит рыбу",
		["Herbalism"] = "Травничество",
		["Mining"] = "Горное дело",
		["BG"] = "поле битвы",
		["Raid"] = "набег",
		["man"] = "чел",
	},
}

local function L(tag)
	if translations[GAME_LOCALE] then
		return translations[GAME_LOCALE][tag] or tag
	else
		return tag
	end
end

local function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

local function getFontSizeText(fontSize)
	if fontSize == 0 then return "Font Size: Small" end
	if fontSize == 2 then return "Font Size: Large" end
	return "Font Size: Medium" 
end

local function getScale(fontSize)
	if fontSize == 0 then return 0.4, 0.2 end
	if fontSize == 2 then return 0.8, 0.4 end
	return 0.6, 0.3
end

local function ZoneInfoPanel_Close()
	Settings.ShowBackground = ZoneInfoClassic_chkShowBackground:GetChecked()
	Settings.BackgroundOpacity = ZoneInfoClassic_sldBackgroundOpacity:GetValue() / 100
	Settings.FontSize = UIDropDownMenu_GetSelectedValue(ZoneInfoClassic_cmbFontSize)
	Settings.ShowFishing = ZoneInfoClassic_chkShowFishing:GetChecked()
	Settings.ShowTransports = ZoneInfoClassic_chkShowTransports:GetChecked()
	Settings.ShowRoadConnections = ZoneInfoClassic_chkShowRoadConnections:GetChecked()
	Settings.ShowInstances = ZoneInfoClassic_chkShowInstances:GetChecked()
	Settings.ShowFlightnodes = ZoneInfoClassic_chkShowFlightnodes:GetChecked()
	Settings.ShowHerbs = ZoneInfoClassic_chkShowHerbs:GetChecked()
	Settings.ShowMiningNodes = ZoneInfoClassic_chkShowMiningNodes:GetChecked()

	if Settings.ShowBackground == true then
		ZoneInfoDataProviderMixin.frame.texture:Show()
	else
		ZoneInfoDataProviderMixin.frame.texture:Hide()
	end
	
	ZoneInfoDataProviderMixin.frame.texture:SetVertexColor(0,0,0,Settings.BackgroundOpacity)

	local headerScale, detailsScale = getScale(Settings.FontSize)
	ZoneInfoDataProviderMixin.headerText:SetScale(headerScale)
	ZoneInfoDataProviderMixin.detailsText:SetScale(detailsScale)
	ZoneInfoClassic_cmdShowFrame:SetScale(headerScale)
end

local function ZoneInfoPanel_Cancel()
	-- Reset options UI
	ZoneInfoClassic_chkShowBackground:SetChecked(Settings.ShowBackground)
	ZoneInfoClassic_sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	UIDropDownMenu_SetSelectedValue(ZoneInfoClassic_cmbFontSize, Settings.FontSize)
	ZoneInfoClassic_chkShowFishing:SetChecked(Settings.ShowFishing)
	ZoneInfoClassic_chkShowTransports:SetChecked(Settings.ShowTransports)
	ZoneInfoClassic_chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)
	ZoneInfoClassic_chkShowInstances:SetChecked(Settings.ShowInstances)
	ZoneInfoClassic_chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)
	ZoneInfoClassic_chkShowHerbs:SetChecked(Settings.ShowHerbs)
	ZoneInfoClassic_chkShowMiningNodes:SetChecked(Settings.ShowMiningNodes)
end

local function cmbFontSizeText_OnClick(self, arg1, arg2, checked)
	UIDropDownMenu_SetSelectedValue(ZoneInfoClassic_cmbFontSize, self.value)
end




function ZoneInfo:OnInitialize()
	self.name = L("ZoneInfo")
	self.title = L("ZoneInfo")

	-- Set defaults
	if Settings == nil then Settings = {} end
	if Settings.ShowBackground == nil then Settings.ShowBackground = true end
	if Settings.BackgroundOpacity == nil then Settings.BackgroundOpacity = 0.4 end
	if Settings.FontSize == nil then Settings.FontSize = 1 end
	if Settings.ShowFishing  == nil then Settings.ShowFishing = true end
	if Settings.ShowTransports  == nil then Settings.ShowTransports = true end
	if Settings.ShowRoadConnections  == nil then Settings.ShowRoadConnections = true end
	if Settings.ShowInstances  == nil then Settings.ShowInstances = true end
	if Settings.ShowFlightnodes  == nil then Settings.ShowFlightnodes = true end
	if Settings.ShowHerbs  == nil then Settings.ShowHerbs = true end
	if Settings.ShowMiningNodes  == nil then Settings.ShowMiningNodes = true end
	
	-- Register a panel in the Interface Addon Options GUI
	ZoneInfo.panel = CreateFrame( "Frame", "ZoneInfoClassicPanel", UIParent );
	ZoneInfo.panel.name = L("ZoneInfo");
	ZoneInfo.panel.okay = function (self) ZoneInfoPanel_Close() end
	ZoneInfo.panel.cancel = function (self) ZoneInfoPanel_Cancel() end
	
	-- Setting: Show Background
	local chkShowBackground = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowBackground", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowBackground:SetPoint("TOPLEFT", 20, -20)
	ZoneInfoClassic_chkShowBackgroundText:SetText(" Show background")
	chkShowBackground.tooltip = "Displays the Zone Information on a semi-transparent background"
	chkShowBackground:SetChecked(Settings.ShowBackground)
	
	-- Setting: Background Opacity
	local sldBackgroundOpacity = CreateFrame("Slider", "ZoneInfoClassic_sldBackgroundOpacity", ZoneInfo.panel, "OptionsSliderTemplate")
	sldBackgroundOpacity:SetPoint("TOPLEFT", 20, -60)
	sldBackgroundOpacity:SetOrientation('HORIZONTAL')
	sldBackgroundOpacity:SetWidth(180)
	sldBackgroundOpacity:SetHeight(20)
	ZoneInfoClassic_sldBackgroundOpacityLow:SetText("0%")
	ZoneInfoClassic_sldBackgroundOpacityHigh:SetText("100%")
	ZoneInfoClassic_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(Settings.BackgroundOpacity * 100, 0)).."%)" )
	sldBackgroundOpacity.tooltipText = "Adjust the opacity of the background, where 0% is transparent and 100% is solid black"
	sldBackgroundOpacity:SetMinMaxValues(0, 100)
	sldBackgroundOpacity:SetValue(Settings.BackgroundOpacity * 100)
	sldBackgroundOpacity:SetScript("OnValueChanged", 
		function(self, value)
			ZoneInfoClassic_sldBackgroundOpacityText:SetText("Background opacity ("..tostring(round(value, 0)).."%)" )
		end);	
		
	-- Setting: Font Size
	local cmbFontSizeText = CreateFrame("FRAME", "ZoneInfoClassic_cmbFontSize", ZoneInfo.panel, "UIDropDownMenuTemplate")
	cmbFontSizeText:SetPoint("TOPLEFT", 0, -100)
	UIDropDownMenu_SetWidth(cmbFontSizeText, 170)
	UIDropDownMenu_Initialize(cmbFontSizeText, 
		function()
			local info0 = UIDropDownMenu_CreateInfo()
			info0.func = cmbFontSizeText_OnClick
			-- Small
			info0.text, info0.value = getFontSizeText(0), 0
			UIDropDownMenu_AddButton(info0)
			-- Medium
			local info1 = UIDropDownMenu_CreateInfo()
			info1.func = cmbFontSizeText_OnClick
			info1.text, info1.value = getFontSizeText(1), 1
			UIDropDownMenu_AddButton(info1)
			-- Large
			local info2 = UIDropDownMenu_CreateInfo()
			info2.func = cmbFontSizeText_OnClick
			info2.text, info2.value = getFontSizeText(2), 2
			UIDropDownMenu_AddButton(info2)
		end)	
	UIDropDownMenu_SetSelectedValue(ZoneInfoClassic_cmbFontSize, Settings.FontSize)
		
	-- Setting: Show Fishing
	local chkShowFishing = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowFishing", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFishing:SetPoint("TOPLEFT", 20, -140)
	ZoneInfoClassic_chkShowFishingText:SetText(" Show fishing skills")
	chkShowFishing.tooltip = "Displays the minimum skill required to fish, the skill required to avoid get-aways, and your current skill"
	chkShowFishing:SetChecked(Settings.ShowFishing)		

	-- Setting: Show Transports
	local chkShowTransports = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowTransports", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowTransports:SetPoint("TOPLEFT", 20, -170)
	ZoneInfoClassic_chkShowTransportsText:SetText(" Show transports")
	chkShowTransports.tooltip = "Displays a list of transports like boats and zeppelins"
	chkShowTransports:SetChecked(Settings.ShowTransports)	

	-- Setting: Show Road Connections
	local chkShowRoadConnections = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowRoadConnections", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowRoadConnections:SetPoint("TOPLEFT", 20, -200)
	ZoneInfoClassic_chkShowRoadConnectionsText:SetText(" Show road connections")
	chkShowRoadConnections.tooltip = "Displays connections with adjacent zones by road"
	chkShowRoadConnections:SetChecked(Settings.ShowRoadConnections)	
	
	-- Setting: Show Instances
	local chkShowInstances = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowInstances", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowInstances:SetPoint("TOPLEFT", 20, -230)
	ZoneInfoClassic_chkShowInstancesText:SetText(" Show instances")
	chkShowInstances.tooltip = "Displays a list of instances with their level and size"
	chkShowInstances:SetChecked(Settings.ShowInstances)		

	-- Setting: Show Flightnodes
	local chkShowFlightnodes = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowFlightnodes", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowFlightnodes:SetPoint("TOPLEFT", 20, -260)
	ZoneInfoClassic_chkShowFlightnodesText:SetText(" Show flight nodes")
	chkShowFlightnodes.tooltip = "Displays a list of flight nodes within the selected zone"
	chkShowFlightnodes:SetChecked(Settings.ShowFlightnodes)		
	
	-- Setting: Show Herbs
	local chkShowHerbs = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowHerbs", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowHerbs:SetPoint("TOPLEFT", 20, -290)
	ZoneInfoClassic_chkShowHerbsText:SetText(" Show herbs")
	chkShowHerbs.tooltip = "Displays a list of herbs found within the selected zone"
	chkShowHerbs:SetChecked(Settings.ShowHerbs)	

	-- Setting: Show MiningNodes
	local chkShowMiningNodes = CreateFrame("CheckButton", "ZoneInfoClassic_chkShowMiningNodes", ZoneInfo.panel, "ChatConfigCheckButtonTemplate");
	chkShowMiningNodes:SetPoint("TOPLEFT", 20, -320)
	ZoneInfoClassic_chkShowMiningNodesText:SetText(" Show mining nodes")
	chkShowMiningNodes.tooltip = "Displays a list of mining nodes found within the selected zone"
	chkShowMiningNodes:SetChecked(Settings.ShowMiningNodes)	

	
	InterfaceOptions_AddCategory(ZoneInfo.panel);
end

function ZoneInfo:OnEnable()
    WorldMapFrame:AddDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfo:OnDisable()
   WorldMapFrame:RemoveDataProvider(ZoneInfoDataProviderMixin)
end

function ZoneInfoDataProviderMixin:OnAdded(mapCanvas)
    MapCanvasDataProviderMixin.OnAdded(self, mapCanvas)

	if not self.frame then
		self.frame = CreateFrame(
			"Frame",
			nil,
			self:GetMap():GetCanvasContainer()
		)
		
		-- Background
		local bgTexture = self.frame:CreateTexture("ZoneInfoBackground")
		bgTexture:SetTexture("background.blp")
		bgTexture:SetVertexColor(0,0,0, Settings.BackgroundOpacity)
		bgTexture:SetAllPoints(self.frame)
		self.frame.texture = bgTexture
		if Settings.ShowBackground == false then
			self.frame.texture:Hide()
		end
		
		self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
		self.frame.dataProvider = self
		self.frame:SetPoint(
			"TOPLEFT",
			self:GetMap():GetCanvasContainer(),
			10,
			-40
		)
		
		-- Minimize button
		local cmdShowFrame = CreateFrame("Button", "ZoneInfoClassic_cmdShowFrame", self.frame, "OptionsButtonTemplate");
		cmdShowFrame:SetPoint("TOPLEFT", 0, 0)
		cmdShowFrame:SetWidth(20)
		local scale = getScale(Settings.FontSize)
		cmdShowFrame:SetScale(scale)
		ZoneInfoClassic_cmdShowFrameText:SetText("-")
		cmdShowFrame:SetScript("OnClick", 
		function(self, value)
			local scale = getScale(Settings.FontSize)
			showFrame = not showFrame
			if showFrame == true then
				ZoneInfoDataProviderMixin.frame:SetHeight(frameHeight)
				ZoneInfoDataProviderMixin.frame:SetWidth(400 * scale)
				ZoneInfoDataProviderMixin.headerText:Show()
				ZoneInfoDataProviderMixin.detailsText:Show()
				ZoneInfoClassic_cmdShowFrameText:SetText("-")
			else
				ZoneInfoDataProviderMixin.detailsText:Hide()
				ZoneInfoDataProviderMixin.headerText:Hide()
				local scale = getScale(Settings.FontSize)
				ZoneInfoDataProviderMixin.frame:SetHeight(22 * scale)
				ZoneInfoDataProviderMixin.frame:SetWidth(20 * scale)
				ZoneInfoClassic_cmdShowFrameText:SetText("+")
			end
		end);	
		
		
		-- Texts
		local font, size = WorldMapTextFont:GetFont()
		local headerScale, detailsScale = getScale(Settings.FontSize)
		
		self.headerText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
		self.headerText:SetFont(font, size, "OUTLINE")
		self.headerText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 10, -20)
		self.headerText:SetWidth(800)
		self.headerText:SetScale(headerScale)
        self.headerText:SetJustifyH("LEFT")
	
		self.detailsText = self.frame:CreateFontString(nil, "OVERLAY", "WorldMapTextFont")
		self.detailsText:SetFont(font, size, "OUTLINE")
		self.detailsText:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 10, -200)
		self.detailsText:SetWidth(800)
		self.detailsText:SetScale(detailsScale)
        self.detailsText:SetJustifyH("LEFT")
	else
		self.frame:SetParent(self:GetMap():GetCanvasContainer())
	end
	
	self.headerText:SetText("ZoneInfo Classic")
	self.detailsText:SetText("")
	
	self.frame:Show()
	self.headerText:Show()
	self.detailsText:Show()
	DEFAULT_CHAT_FRAME:AddMessage("ZoneInfo Classic loaded.")
end

function ZoneInfoDataProviderMixin:RemoveAllData()
    self.frame:Hide()
end

-- Used by GetFishingSkill
local function FindSpellIDByTexture(texture)
	local id = 1
	local spellTexture = GetSpellTexture(id)
	while (spellTexture) do
		if (spellTexture and spellTexture == texture) then
			return id
		end
		id = id + 1
		spellTexture = GetSpellTexture(id)
	end
	return nil;
end

function ZoneInfoDataProviderMixin:GetFishingSkill()
	local fishingSkill = 0

	-- Get the name of the Fishing skill
	local fishingSkillName = L("Fishing")
	local spellID = FindSpellIDByTexture("Interface\\Icons\\Trade_Fishing")
	if spellID then
		fishingSkillName = GetSpellInfo(spellID);
	end
	
	-- Loop through all lines on the Skills tab of the Character window
	for i = 1, GetNumSkillLines() do
		local name, _, _, skill = GetSkillLineInfo(i)
		if ( name == fishingSkillName ) then
			-- Found Fishing -> exit the loop
			fishingSkill = skill
			break
		end
	end

	return fishingSkillName, fishingSkill or 0 
end

function ZoneInfoDataProviderMixin:GetHerbalismSkill()
	local herbalismSkill = 0

	-- Get the name of the Herbalism skill
	local herbalismSkillName = L("Herbalism")
	local spellID = FindSpellIDByTexture("Interface\\Icons\\Trade_Herbalism")
	if spellID then
		herbalismSkillName = GetSpellInfo(spellID);
	end
	
	-- Loop through all lines on the Skills tab of the Character window
	for i = 1, GetNumSkillLines() do
		local name, _, _, skill = GetSkillLineInfo(i)
		if ( name == herbalismSkillName ) then
			-- Found Herbalism -> exit the loop
			herbalismSkill = skill
			break
		end
	end

	return herbalismSkillName, herbalismSkill or 0 
end

function ZoneInfoDataProviderMixin:GetMiningSkill()
	local miningSkill = 0

	-- Get the name of the Mining skill
	local miningSkillName = L("Mining")
	local spellID = FindSpellIDByTexture("Interface\\Icons\\Trade_Mining")
	if spellID then
		miningSkillName = GetSpellInfo(spellID);
	end
	
	-- Loop through all lines on the Skills tab of the Character window
	for i = 1, GetNumSkillLines() do
		local name, _, _, skill = GetSkillLineInfo(i)
		if ( name == miningSkillName ) then
			-- Found Mining -> exit the loop
			miningSkill = skill
			break
		end
	end

	return miningSkillName, miningSkill or 0 
end


function ZoneInfoDataProviderMixin:RefreshAllData(fromOnShow)
	if not self.frame then
		return
	end

	local currentUiMapID = WorldMapFrame:GetMapID()
	local zoneText = Tourist:GetMapNameByIDAlt(currentUiMapID)
	local playerLevel = UnitLevel("player")

	trace("RefreshAllData: currentUiMapID: "..tostring(currentUiMapID).." = "..tostring(zoneText))

	if currentUiMapID and (Tourist:IsZoneOrInstance(currentUiMapID) or Tourist:DoesZoneHaveInstances(currentUiMapID)) and not Tourist:IsContinent(currentUiMapID) then
		-- HEADER
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		
		local low, high = Tourist:GetLevel(currentUiMapID)	
		local levelText = ""
		if low > 0 and high > 0 then
			levelText = Tourist:GetLevelString(currentUiMapID)
			local r, g, b = Tourist:GetLevelColor(currentUiMapID)
			levelText = string.format("|cff%02x%02x%02x[%s] (%s)|r", r * 255, g * 255, b * 255, levelText, playerLevel)

			local groupSizeString = Tourist:GetInstanceGroupSizeString(currentUiMapID, true)
			if groupSizeString ~= "" then
				levelText = levelText.." "..groupSizeString.." "..L("man")
			end
		end
			
		if levelText == "" then
			self.headerText:SetText(zoneText)
		else
			table.insert(t,zoneText)
			table.insert(t,levelText)
			self.headerText:SetText(table.concat(t, "\n"))
			for k in pairs(t) do
				t[k] = nil
			end
		end

		-- DETAILS
		self.detailsText:SetText("")
		
		-- Determine Fishing level
		local minFish, maxFish
		local fishing_skill_text
		if Settings.ShowFishing then
			minFish, maxFish = Tourist:GetFishingLevel(currentUiMapID)
			if minFish == 0 or not minFish then 
				-- no fishing in this zone
				minFish = nil
			end
			if minFish then
				local fishingSpell, fishingSkill = ZoneInfoDataProviderMixin:GetFishingSkill()
				local r,g,b = 1,1,0
				local r1,g1,b1 = Tourist:CalculateLevelColor(minFish, maxFish, fishingSkill) 
				local levelString = Tourist:GetFishingLevelString(currentUiMapID)					

				fishing_skill_text = string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%s] (%d)|r",r * 255, g * 255, b * 255,fishingSpell, r1 * 255, g1 * 255, b1 * 255, levelString, fishingSkill)

				if not fishing_skill_text then
					minFish = nil
				end
			end
		end
		
		-- Determine Herbalism level
		local herbalismSpell, herbalismSkill
		herbalismSkill = 0
		local hasHerbs = Tourist:DoesZoneHaveHerbs(currentUiMapID) and Settings.ShowHerbs
		if hasHerbs then
			herbalismSpell, herbalismSkill = ZoneInfoDataProviderMixin:GetHerbalismSkill()
			trace(tostring(herbalismSpell)..": "..tostring(herbalismSkill))
		end	
	
		-- Determine Mining level
		local miningSpell, miningSkill
		miningSkill = 0
		local hasMiningNodes = Tourist:DoesZoneHaveMiningNodes(currentUiMapID) and Settings.ShowMiningNodes
		if hasMiningNodes then
			miningSpell, miningSkill = ZoneInfoDataProviderMixin:GetMiningSkill()
			trace(tostring(miningSpell)..": "..tostring(miningSkill))
		end	
	
		-- Determine transport connections
		local transports = {}
		local hasTransports
		local roadConnections = {}
		local hasRoadConnections
		if Settings.ShowTransports or Settings.ShowRoadConnections then
			for path in Tourist:IterateBorderZones(currentUiMapID, false) do
				if not Tourist:IsZoneOrInstance(path) then
					transports[path] = true
					hasTransports = true
				else 
					if Tourist:IsZone(path) then
						roadConnections[path] = true
						hasRoadConnections = true
					end
				end
			end
		end
		local hasFlightnodes = Tourist:DoesZoneHaveFlightnodes(currentUiMapID) and Settings.ShowFlightnodes
		local hasInstances = Tourist:DoesZoneHaveInstances(currentUiMapID) and Settings.ShowInstances
		local noTable = false
		
		-- Add fishing level
		if minFish and fishing_skill_text then
			if hasTransports or hasRoadConnections or hasInstances or hasFlightnodes or hasHerbs or hasMiningNodes then 
				table.insert(t,fishing_skill_text)
				table.insert(t,"")
			else
				-- Fishing only: show one line (no table)
				self.detailsText:SetText(fishing_skill_text)
				noTable = true
			end
		end
		
		-- Add transport connections
		if hasTransports and Settings.ShowTransports then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Transports")))
			for k, v in pairs(transports) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasRoadConnections or hasInstances or hasFlightnodes or hasHerbs or hasMiningNodes then
				table.insert(t,"")
			end
		end

		-- Add road connections
		if hasRoadConnections and Settings.ShowRoadConnections then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Road connections")))
			for k, v in pairs(roadConnections) do
				r, g, b = Tourist:GetFactionColor(k)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..k))
			end
			if hasInstances or hasFlightnodes or hasHerbs or hasMiningNodes then
				table.insert(t,"")
			end
		end
	
		-- Add instances
		if hasInstances then
			table.insert(t, string.format("|cffffffff%s:|r", L("Instances")))
			for instance in Tourist:IterateZoneInstances(currentUiMapID) do
				local complex = nil
				if not Tourist:IsComplex(currentUiMapID) then
					complex = Tourist:GetComplex(instance)
				end
				local low, high = Tourist:GetLevel(instance)
				local r1, g1, b1 = Tourist:GetFactionColor(instance)
				local r2, g2, b2 = Tourist:GetLevelColor(instance)
				local groupSize = Tourist:GetInstanceGroupSize(instance)
				local groupSizeString = Tourist:GetInstanceGroupSizeString(instance, true)
				local name = instance
				if complex then
					name = complex .. " - " .. instance
				end
				if Tourist:IsBattleground(instance) then
					name = L("BG")..": " .. name
				elseif groupSize > 5 then
					name = L("Raid")..": " .. name
				end						
				if low == high then
					if groupSize > 0 then
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
					else
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
					end
				else
					if groupSize > 0 then
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r " .. groupSizeString.." "..L("man"), r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
					else
						table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
					end
				end
			end
			
			if hasFlightnodes or hasHerbs or hasMiningNodes then
				table.insert(t,"")
			end		
		end
		
		-- Add flight nodes
		if hasFlightnodes then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Flight nodes")))
			for node in Tourist:IterateZoneFlightnodes(currentUiMapID) do
				r, g, b = Tourist:GetFlightnodeFactionColor(node.faction)
				table.insert(t, string.format("|cff%02x%02x%02x%s|r", r * 255, g * 255, b * 255, "- "..node.name)) --.." ("..tostring(node.factionName)..")"))
			end
			
			if hasHerbs or hasMiningNodes then
				table.insert(t,"")
			end					
		end
		
		-- Add herbs
		if hasHerbs then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Herbalism").." ("..tostring(herbalismSkill)..")"))
			for herb in Tourist:IterateHerbsByZone(currentUiMapID) do
				r, g, b = Tourist:GetGatheringSkillColor(herb.minLevel, herbalismSkill)
				table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r ".." x"..tostring(herb.numNodes), 255, 255, 0, tostring(herb.name), r * 255, g * 255, b * 255, tostring(herb.minLevel)))
			end
			
			if hasMiningNodes then
				table.insert(t,"")
			end								
		end

		-- Add miningNodes
		if hasMiningNodes then
			local r, g, b
			table.insert(t, string.format("|cffffffff%s:|r", L("Mining").." ("..tostring(miningSkill)..")"))
			for miningNode in Tourist:IterateMiningNodesByZone(currentUiMapID) do
				r, g, b = Tourist:GetGatheringSkillColor(miningNode.minLevel, miningSkill)
				table.insert(t, "- "..string.format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r ".." x"..tostring(miningNode.numNodes), 255, 255, 0, tostring(miningNode.nodeName), r * 255, g * 255, b * 255, tostring(miningNode.minLevel)))
			end
		end
		
		-- Show result (if table)
		if noTable == false then
			self.detailsText:SetText(table.concat(t, "\n"))
			for k in pairs(t) do
				t[k] = nil
			end
		end				

	elseif not currentUiMapID then
		-- No map ID
		self.headerText:SetTextColor(1,1,1)
		self.headerText:SetText("??")
		self.detailsText:SetText("")
	else
		-- Continent, World map, Cosmic map
		self.headerText:SetTextColor(Tourist:GetFactionColor(currentUiMapID))
		self.headerText:SetText(zoneText)
		self.detailsText:SetText("")		
	end
	
	-- Set frame height and width
	local headerScale, detailsScale = getScale(Settings.FontSize)
	if self.detailsText:GetText() == nil then
		frameHeight = (self.headerText:GetHeight() + 45) * headerScale
	else
		frameHeight = (self.detailsText:GetHeight() + 225) * detailsScale 	
	end
	if showFrame == true then
		self.frame:SetHeight(frameHeight)
		self.frame:SetWidth(headerScale * 400)
	end
end


